from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.utils import timezone

import datetime

from downtime.models import Period

class Command(BaseCommand):
    help = 'Start downtime due to maintenance/deployment/whatever.'

    def handle(self, *args, **options):
        p = Period(enabled=True, start_time=timezone.now())
        p.save()
        self.stdout.write('Successfully started downtime')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Period',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('start_time', models.DateTimeField(null=True, blank=True)),
                ('end_time', models.DateTimeField(null=True, blank=True)),
                ('enabled', models.BooleanField(default=False)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



import datetime

from django.db.models import Manager
from django.db.models.query import QuerySet
from django.conf import settings
from django.utils import timezone


class PeriodQuerySet(QuerySet):

    def active(self):
        return self.filter(enabled=True)

    def is_deployment(self):
        return self.filter(start_time__lte=timezone.now(), end_time=None)

    def is_down(self):
        return self.filter(start_time__lte=timezone.now(),
                           end_time__gte=timezone.now())


class PeriodManager(Manager):
    def get_queryset(self):
        return PeriodQuerySet(self.model, using=self._db)

    def active(self):
        return self.get_queryset().active()
    
    def is_deployment(self):
        return self.get_queryset().active().is_deployment()
        
    def is_down(self):
        return self.get_queryset().active().is_down() or self.get_queryset().active().is_deployment()



from django.shortcuts import render, redirect
from django.conf import settings

from downtime.models import Period

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object


class DowntimeMiddleware(MiddlewareMixin):
    def process_request(self, request):
        exempt_exact_urls = getattr(settings,
                                    'DOWNTIME_EXEMPT_EXACT_URLS', None)
        if exempt_exact_urls:
            for url in exempt_exact_urls:
                if request.path == url:
                    return None

        exempt_paths = getattr(settings, 'DOWNTIME_EXEMPT_PATHS', ('/admin',))
        for path in exempt_paths:
            if request.path.startswith(path):
                return None

        objects = Period.objects.is_down()

        if objects.count():
            # we are down.
            url_redirect = getattr(settings, 'DOWNTIME_URL_REDIRECT', None)
            if url_redirect:
                return redirect(url_redirect)
            else:
                return render(request, "downtime/downtime.html", status=503)



import datetime

from django.utils import timezone
import factory

from downtime.models import Period


class PeriodFactory(factory.DjangoModelFactory):
    start_time = timezone.now()
    end_time = timezone.now() + datetime.timedelta(days=2)
    enabled = True

    class Meta:
        model = Period



from mock import Mock

from django.test import TestCase


from downtime.middleware import DowntimeMiddleware
from downtime.tests.factories import PeriodFactory


class DowntimeMiddlewareTest(TestCase):

    def setUp(self):
        self.dm = DowntimeMiddleware()
        self.request = Mock()
        self.request.session = {}
        self.period = PeriodFactory.create()

    def test_exempt_exact_urls(self):
        self.request.path = '/test/page/id'
        self.assertTrue(self.dm.process_request(self.request))

        with self.settings(DOWNTIME_EXEMPT_EXACT_URLS=('/test/page/id', )):
            self.assertIsNone(self.dm.process_request(self.request))

            self.request.path = '/test/page/id/child'
            self.assertTrue(self.dm.process_request(self.request),
                            'Children pages are not exempted to exact urls')

    def test_exempt_paths(self):
        self.request.path = '/admin'
        self.assertIsNone(self.dm.process_request(self.request))

    def test_render(self):
        self.request.path = '/'

        response = self.dm.process_request(self.request)
        self.assertEqual(response.status_code, 503)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.utils import timezone

import datetime

from downtime.models import Period

class Command(BaseCommand):
    help = 'End downtime after finished maintenance/deployment/whatever.'

    def handle(self, *args, **options):
        objects = Period.objects.is_deployment()
        _now = timezone.now()
        for obj in objects:
            obj.end_time = _now
            obj.save()

        self.stdout.write('Successfully ended downtime')
